@@ -4,17 +4,18 @@ from .manage_file import create_all_parents, searching_all_files, unzip, pbar
 from . import load_google_drive
 from . import google
 from . import stutils
-from . import meteo
+from . import meteo as stmeteo
 from . import easy_import
 # from .pickle_wrapper import load_pickle, dump_pickle
 from .regress2 import regress2
+from .lins_ccc_1989 import concordance_correlation_coefficient
 
 __all__ = [
     "Yaml", 
     "Montre", 
     "create_all_parents", "searching_all_files", "unzip", "pbar",
-    "load_google_drive", "google",
+    "load_google_drive", "google", "stmeteo",
     # "load_pickle", "dump_pickle",
-    "stutils", "regress2",
+    "stutils", "regress2", 'concordance_correlation_coefficient',
     "easy_import"
     ]
\ No newline at end of file
