@@ -1,9 +1,9 @@
 # mount the drive
-def mount_drive():
+def mount_drive(force_remount = False):
     from google.colab import drive
     from pathlib import Path
   
-    drive.mount('/content/drive')
+    drive.mount('/content/drive', force_remount = force_remount)
     return Path.cwd().joinpath('drive/My Drive')
 
 # download from colab
@@ -30,4 +30,17 @@ def download_file(src, filename, **kwargs):
     else:
         with open(filename, "wb") as f:
             pickle.dump(src, f)
-    files.download(filename)
\ No newline at end of file
+    files.download(filename)
+
+def check_colab_gpu():
+    import os
+    if int(os.environ["COLAB_GPU"]) > 0:
+        print("A GPU is connected.")
+        gpu = 1
+    elif "COLAB_TPU_ADDR" in os.environ and os.environ["COLAB_TPU_ADDR"]:
+        print("A TPU is connected.")
+        gpu = 2
+    else:
+        print("No accelerator is connected.")
+        gpu = 0
+    return gpu
\ No newline at end of file
