@@ -4,10 +4,9 @@ import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
 from scipy import stats
-from datetime import datetime
+from datetime import datetime, timedelta
 from sklearn.metrics import mean_squared_error
 
-
 def pprint(values, p = 2):
     try:
         len(values)
@@ -145,8 +144,8 @@ def stats_measures_df(df, name1, name2, return_dict = False):
     else:
         return [r2, slope, rmse, mbe]
 
-def load_csv(p, fmt = 'yearfirst', index_col = 0, strip_cols = True, duplicated_time = True, missing = -9999., columns = None):
-    df = pd.read_csv(p, index_col = index_col)
+def load_csv(p, fmt = 'yearfirst', index_col = 0, strip_cols = True, duplicated_time = True, missing = -9999., columns = None, **kwargs):
+    df = pd.read_csv(p, index_col = index_col, **kwargs)
     if strip_cols:
         df.columns = [c.strip() for c in df.columns]
     if fmt: 
@@ -216,11 +215,20 @@ def rotate_ticks(ax, which, degree):
     ax.tick_params(axis=which, rotation=degree)
 
 def sort_list_by(lista, listb):
+    # sort lista by listb
     lista = [i for _, i in sorted(zip(listb, lista))]
     return lista
 
-def add_text(ax, x, y, text):
-    ax.text(x, y, text, transform = ax.transAxes)
+def add_text(ax, x, y, text, horizontalalignment = 'center', verticalalignment = 'center'):
+    ax.text(x, y, text, transform = ax.transAxes, horizontalalignment = horizontalalignment, verticalalignment = verticalalignment)
+
+def add_line(ax, loc, linestyle = '--', color = 'k', alpha = 0.5, direction = 'h', bmin = 0, bmax = 1):
+    if direction.lower() in ['h', 'horizontal']:
+        ax.axhline(loc, linestyle = linestyle, color = color, alpha = alpha, xmin = bmin, xmax = bmax)
+    elif direction.lower() in ['v', 'vertical']:
+        ax.axvline(loc, linestyle = linestyle, color = color, alpha = alpha, ymin = bmin, ymax = bmax)
+    else:
+        raise Exception('Options: h, horizontal, v, and vertical!')
 
 def shift_axis_label(ax, which, x_shift, y_shift):
     if which == 'x':
@@ -249,4 +257,119 @@ def reset_sci_env():
 def savefig(fig, savefile, dpi = 600, bbox_inches = "tight", transparent = False, **kwargs):
     fig.savefig(savefile, dpi = dpi, bbox_inches = bbox_inches, transparent = transparent, **kwargs)
 
-colors = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
\ No newline at end of file
+def str2date(str_, format):
+    return datetime.strptime(str_, format)
+
+def date2str(date, format):
+    return date.strftime(format)
+
+def timedif(dt1, dt2, mode = 'None'):
+    dif = dt2 - dt1
+    if mode.lower in ['s', 'sec', 'second']:
+        return dif.total_seconds()
+    elif mode.lower in ['min', 'minute']:
+        return dif.total_seconds() / 60
+    elif mode.lower in ['h', 'hour']:
+        return dif.total_seconds() / 60 / 60
+    elif mode.lower in ['d', 'day']:
+        return dif.total_seconds() / 60 / 60 / 24
+    elif mode.lower in ['mon', 'month']:
+        return dif.total_seconds() / 60 / 60 / 24 / 30
+    elif mode.lower in ['y', 'yr', 'year']:
+        return dif.total_seconds() / 60 / 60 / 24 / 365
+    else:
+        return dif
+
+def create_folder(des):
+    # exist_ok = False: DO NOT make if the directory exists!
+    try:
+        des.mkdir(mode = 0o777, parents = True, exist_ok = False)
+        print(f'Directory made: {des} ')
+    except FileExistsError as e:
+        print('Not creating, target directory exists!')
+        # FileExistsError
+
+def format_axis_datetime(ax, fmt = '%m/%Y', which = 'x'):
+    import matplotlib.dates as mdates
+    myFmt = mdates.DateFormatter(fmt)
+    if which == 'x':
+        ax.xaxis.set_major_formatter(myFmt)
+    else:
+        ax.yaxis.set_major_formatter(myFmt)
+
+def nrow_x_ncols(acnt):
+    # auto nrows and ncols
+    nc = int(np.ceil(np.sqrt(acnt)))
+    if nc*(nc-1) >= acnt:
+        nr = nc - 1
+    else:
+        nr = nc
+    return nr, nc
+
+def drop_duplicated_row_cols(df, axis):
+    df = df.copy()
+    if axis == 0:
+        # drop rows with duplicated index
+        df = df[~df.index.duplicated(keep='first')]
+    else:
+        df = df.loc[:,~df.columns.duplicated()]
+    return df
+
+def intersect_lists(d, method = 1):
+    if method == 1:
+        # Intersection of multiple lists
+        # d = [list1, list2, ...]
+        # solution1:
+        d = list(set.intersection(*map(set,d)))
+    else:
+        # solution2:
+        from functools import reduce
+        # apply intersect1d to (a list of) multiple lists:
+        d = list(reduce(np.intersect1d, d))
+    return d
+
+def drop_all_vals(df, val = 0, axis = 1):
+    # default: drop rows all equal zero
+    df = df.copy()
+    return df.loc[~(df==val).all(axis=axis)]
+
+
+def df_replace_dict(df, column, dict_):
+    # Replace columns values by dict
+    # example: dict_ = {1: 'winter', 2: 'spring', 3: 'summer', 4: 'autumn'}
+    # df["seasons"] = df["seasons"].map(dict_)
+    df = df.copy()
+    df[column] = df[column].map(dict_)
+    return df
+
+def get_handles_labels(ax):
+    handles, labels = ax.get_legend_handles_labels()
+    # by_label = dict(zip(labels, handles))
+    return handles, labels
+
+def reorder_labels(handles, labels, ncol):
+    leng = len(labels)
+    nrow = np.ceil(len(labels) / ncol).astype(int)
+    labels_new = labels + [0 for i in range(nrow * ncol - leng)]
+    labels_new[0: len(labels)] = labels
+    labels_new = np.array(labels_new).reshape(nrow, ncol).T.ravel()
+    idx = np.where(labels_new != '0')
+    labels_new = labels_new[idx]
+    
+    handles_new = handles + [0 for i in range(nrow * ncol - leng)]
+    handles_new[0: len(handles)] = handles
+    handles_new = np.array(handles_new).reshape(nrow, ncol).T.ravel()
+    
+    idx = np.where(handles_new != 0)
+    handles_new = handles_new[idx]
+    
+    # labels_new = np.resize(np.array(labels), (nrow, ncol)).T
+    # handles_new = np.resize(np.array(handles), (nrow, ncol)).T
+    # labels_new = np.unique(labels_new.ravel())
+    # # handles_new = np.unique(handles_new.ravel())
+    
+    return handles_new, labels_new
+
+def get_quantile_index(s, q):
+    # OR: s[s == s.quantile(.5, interpolation='lower')]
+    return (s.sort_values()[::-1] <= s.quantile(.5)).idxmax()
\ No newline at end of file
